/******************************************************************//**
 * Zerocat Chipflasher --- Flash free firmware to BIOS chips,
 *                         kick the Manageability Engine.
 *
 * Copyright (C) 2015, 2016, 2017, 2018 Kai Mertens <kmx@posteo.net>
 *
 * The Zerocat Chipflasher is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *
 * The Zerocat Chipflasher is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This file is part of the Zerocat Chipflasher.
 *
 *
 * @file
 *
 */

#ifndef __CHIPSPEC_H__
#define __CHIPSPEC_H__

/**
 * @defgroup ChipspecData Chip Specification
 *
 * @brief
 * Objects that help to manage chip specifications.
 *
 */

/**
 * @defgroup MemSizes Common Memory Sizes
 *
 * @brief
 * Shortcuts for commonly used power of two values.
 *
 */

/**
 * @defgroup ID_JEDEC JEDEC Chip IDs
 * @brief Internal chip IDs according to JEDEC standard.
 *
 */

/**
 * @defgroup SPI-Command-IDs SPI Command IDs
 *
 * @brief
 * These IDs are bit numbers that help to generate bit values.
 *
 * @note
 * Take care to not exceed index 31!
 *
 */



/**
 * @ingroup ChipspecData
 * @{
 */
#define ARRAY_CHIPSPEC    13     //!< Number of entries in #chipspec database.
#define CHIPSPEC_DEFAULT  0
/** @} */



/**
 * @ingroup MemSizes
 * @{
 */
#define SIZE_1            0x00000001
#define SIZE_2            0x00000002
#define SIZE_4            0x00000004
#define SIZE_8            0x00000008
#define SIZE_16           0x00000010
#define SIZE_32           0x00000020
#define SIZE_64           0x00000040  //!< Common Size of OTP
#define SIZE_256          0x00000100  //!< Usual page size
#define SIZE_1K           0x00000400
#define SIZE_4K           0x00001000  //!< Size of 4K-sector
#define SIZE_16K          0x00004000
#define SIZE_32K          0x00008000  //!< Size of 32K-block
#define SIZE_64K          0x00010000  //!< Size of 64K-block
#define SIZE_4MBIT        0x00080000  //!< Chipsize  4Mbit
#define SIZE_8MBIT        0x00100000  //!< Chipsize  8Mbit
#define SIZE_16MBIT       0x00200000  //!< Chipsize 16Mbit
#define SIZE_32MBIT       0x00400000  //!< Chipsize 32Mbit
#define SIZE_64MBIT       0x00800000  //!< Chipsize 64Mbit
#define SIZE_128MBIT      0x01000000  //!< Chipsize 128Mbit
#define SIZE_256MBIT      0x02000000  //!< Chipsize 256Mbit
/** @} */



/**
 * @ingroup ID_JEDEC
 * @{
 */
#define ID_JEDEC_AT26DF321        0x001f4700
#define ID_JEDEC_AT26DF161        0x001f4600
/** @} */



/**
 * @ingroup SPI-Command-IDs
 * @{
 */
//Read Commands
#define X03_READ            0
#define X0B_READHF          1
//Program and Erase Commands
#define X20_SE              2   //Sector Erase (4K)
#define X52_BE32K           3   //Block Erase (32K)
#define XD8_BE64K           4   //Block Erase (64K)
#define X60_CE              5   //Full Chip Erase
#define XC7_CE              6   //Full Chip Erase
#define X02_PP              7   //Page Program
#define XAD_CP              8   //Continuous Program
#define X02_BP              9   //Single Byte Program
//Protection Commands
#define X06_WREN            10  //Write Enable
#define X04_WRDI            11  //Write Disable
#define X50_EWSR            12  //Enable Write Status Register
//Status Register Commands
#define X05_RDSR            13  //Read Status Register
#define X01_WRSR            14  //Write Status Register
//Our NOP Flag
//FIXME: We should find another way.
#define X00_NOP             15  //if this is set, a NOP is send to the device after X9F_RDID.
//Miscellaneous Commands
#define X9F_RDID            16  //Read JEDEC Manufacturer and Device ID (3 Databytes)
#define XB9_DP              17  //Enter Deep Powerdown
#define XAB_RDP             18  //Release from Deep Powerdown
//Some Extras
#define XAB_RES             19  //Read Electronic Signature (1 Databyte)
#define X90_REMS            20  //Read Manufacturer and Device ID (2 Databytes)
#define X3C_RSPR            21  //Read Sector Protection Registers
#define X36_PS              22  //Protect Sector Command (Block 4K)
#define X39_US              23  //Unprotect Sector Command (Block 4K)
#define X70_ESRY            24  //Enable Ready at Pin
#define X80_DSRY            25  //Disable Ready at Pin
#define X35_RDSR2           26  //Read Status Register 2
#define X15_RDSR3           27  //Read Status Register 3
#define X31_WRSR2           28  //Write Status Register 2
#define X11_WRSR3           29  //Write Status Register 3
/** @} */



/**
 * @ingroup ChipspecData
 *
 * @brief
 * Chip specifications, provided within the source code.
 *
 */
struct tag_chipspec {
  char name[15];                        //!< chip's name, i.e. MX25L1605D
  const int id_JEDEC;                   //!< If not applicable or unknown, clear cmdset.X9F_RDID.
  const unsigned int chipsize;          //!< Size of the chip.
  const int pagesize;                   //!< If not applicable or unknown, clear cmdset.X02_PP.
  const signed char sizeOTP;            //!< If not applicable or unknown, clear cmdset.X2B_RDSCUR.
  const int cmdset;                     //!< selected commands
  const unsigned char sr_is_static[3];  //!< True, if Status Register is not initialized during Power-Up
  const unsigned char sr_wrmask[3];     //!< maskout unwritable bits of the status register
  char *sr_bitnames[3];                 //!< Status Register 0 Bit Names
};

#endif
/* __CHIPSPEC_H__ */
