/******************************************************************//**
 * Zerocat Chipflasher --- Flash free firmware to BIOS chips,
 *                         kick the Manageability Engine.
 *
 * Copyright (C) 2015, 2016, 2017, 2018 Kai Mertens <kmx@posteo.net>
 *
 * The Zerocat Chipflasher is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *
 * The Zerocat Chipflasher is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This file is part of the Zerocat Chipflasher.
 *
 *
 * @file
 *
 * @brief
 * Principle file of the chipflasher's host software.
 *
 */

//standard library headers
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/timerfd.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <signal.h>
#include <errno.h>

//project headers
#include "../../firmware/src/libcommon/common.h"
#include "../../firmware/src/libcommon/filespec.h"
#include "../../firmware/src/libcommon/serial-codes.h"        //include serial control codes
#include "connect.h"

//project source files
#include "../../firmware/src/libcommon/hexdigit2bin.c"
#include "../../firmware/src/libcommon/bin2hexdigit.c"
#include "../../firmware/src/libcommon/MOT_addrlen.c"
#include "../../firmware/src/libcommon/line_info.c"

//globals
struct termios old_stdio;
char *fileToSend = FILE_TO_SEND;
char *fileToCatch = FILE_TO_CATCH;



/******************************************************************//**
 * @brief Validate error code.
 * @param errcode Error code.
 * @param retry_count Number of additional line requests.
 * @todo Shouldn't we use stderr for error messages?
 *
 */
void put_error (
  char errcode,
  int retry_count
  )
{
  if(errcode != ERRC__SUCCESS) {
    printf(MSG_ERROR);
    headline();
    printf("\rTransmission Error: ");
  }
  else {
    if(!retry_count)
      printf(MSG_START);
    else
      printf(MSG_WARN);
    headline();
  }

  switch(errcode) {
    case ERRC__SUCCESS:
      if(!retry_count) {
        printf("\rTransmission complete, please verify.\r\n");
      }
      else {
        printf("\rTransmission complete, however ");
        (retry_count == 1) ? printf("one transmission error ") :
        printf("%d transmission errors ", retry_count);
        printf("had to be repaired. Please verify!\r\n");
      }
      break;

    case ERRC__LINE_COUNT_MISMATCH:
      printf("Line Count Mismatch.\r\n");
      break;

    case ERRC__CHECKSUM_MISMATCH:
      printf("Checksum Mismatch.\r\n");
      break;

    case ERRC__JOB_CANCELLATION:
      printf("Job Cancellation.\r\n");
      break;

    case ERRC__NO_MOT:
      printf("No Motorola S-Record.\r\n");
      break;

    case ERRC__BUFFER_OVERRUN:
      printf("Buffer Overrun.\r\n");
      break;

    case ERRC__LINE_TOO_LONG:
      printf("Line too Long.\r\n");
      break;

    case ERRC__NO_HEX_DIGIT:
      printf("No HEX Digit received.\r\n");
      break;

    case ERRC__LINE_ENDING_ERROR:
      printf("Line Ending Error.\r\n");
      break;

    case ERRC__HEXDUMP_PARSING_ERROR:
      printf("Hexdump Parsing Error.\r\n");
      break;

    case ERRC__OUT_OF_CHARSET:
      printf("Character out of Charset.\r\n");
      break;

    case ERRC__LINE_TIMEOUT:
      printf("Line Timeout.\r\n");
      break;

    case ERRC__LINE_LENGTH_MISMATCH:
      printf("Line Length Mismatch.\r\n");
      break;

    case ERRC__NO_SUCH_FILE:
      printf("File '%s' is not available.\r\n", fileToSend);
      break;

    case ERRC__PORT_OPEN_FAILURE:
      printf("Failed to Open TTY Port.\r\n");
      printf("An existing TTY port connection now has probably been corrupted, sorry.\r\n");
      break;
  }
  printf(MSG_STOP);
}



/******************************************************************//**
 * @brief
 * Asserts #MARK or #SPACE to the DTR line.
 *
 * @param fdtty
 * File descriptor for serial device Teletype.
 * @param level
 * i.e. #MARK or #SPACE.
 *
 */
void set_DTR (
  int fdtty,
  int level
  )
{
  int serial;

  //get struct info
  ioctl(fdtty, TIOCMGET, &serial);

  //modify struct
  if(level == MARK)   //negative voltage on DTR
    serial &= ~TIOCM_DTR;
  else    //positive voltage on DTR
    serial |= TIOCM_DTR;

  //apply struct info
  ioctl(fdtty, TIOCMSET, &serial);
}


/**
 * Lock Serial Port
 *
 * See `man tty_ioctl`
 *
 */
void lock_tty(
  int fdtty
)
{
  ioctl(fdtty, TIOCEXCL);
}



/**
 * Unlock Serial Port
 *
 * See `man tty_ioctl`
 *
 */
void unlock_tty(
  int fdtty
)
{
  ioctl(fdtty, TIOCNXCL);
}



/******************************************************************//**
 * @brief
 * Asserts #MARK or #SPACE to the RTS line.
 *
 * @param fdtty
 * File descriptor for serial device Teletype.
 * @param level
 * i.e. #MARK or #SPACE.
 *
 */
void set_RTS (
  int fdtty,
  int level
  )
{
  int serial;

  //get struct info
  ioctl(fdtty, TIOCMGET, &serial);

  //modify struct
  if(level == MARK)   //negative voltage on RTS
    serial &= ~TIOCM_RTS;
  else    //positive voltage on RTS
    serial |= TIOCM_RTS;

  //apply struct info
  ioctl(fdtty, TIOCMSET, &serial);
}



/******************************************************************//**
 * @brief
 * Helps to get the current status of the DTR line. This code had
 * been picked from the Linux Programmer's Manual.
 *
 * @note
 * Please refer to `$ man tty_ioctl`, section 'modem control'.
 *
 * @return
 * Returns zero if DTR is set, otherwise 1.
 *
 */
int get_DTR (
  int fdtty
  )
{
  int serial;

  ioctl(fdtty, TIOCMGET, &serial);

  if(serial & TIOCM_DTR)
    puts(MSG_START"TIOCM_DTR is not set.\r"MSG_STOP);
  else
    puts(MSG_START"TIOCM_DTR is set.\r"MSG_STOP);

  return(serial & TIOCM_DTR);
}



void headline (
  void
  )
{
  int n;

  for(n = 5; n > 0; n--)
    putchar('~');
  printf(HEADER_CONNECT);
  for(n = MENUWIDTH - (sizeof(HEADER_CONNECT) - 1) - 5; n > 0; n--)
    putchar('~');
  puts("\r");
}



/******************************************************************//**
 * @brief
 * Put a greeting with some basic info on screen.
 *
 */
void greeting (
  const char *port,
  const char *rate
  )
{
  printf(HEADER_LICENSE);
  printf(MSG_START);
  headline();
  printf("This is connect, the host software.\r\n\n");
  printf(" * Connection Specs\r\n");
  printf("   TTY Port:\r\t\t%s\r\n", port);
  printf("   Baudrate:\r\t\t%s\r\n", rate);
  printf("   Please disable network devices and resource demanding processes\r\n");
  printf("   in order to obtain a reliable serial connection.\r\n\n");
  printf(" * Chip Readouts\r\n");
  printf("   File in use to store chip readouts:\r\t\t\t\t\t%s\r\n", fileToCatch);
  printf("   Please backup your data before initiating a chip readout!\r\n\n");
  printf(" * Chip Flashing\r\n");
  printf("   File in use for chip flashing:\r\t\t\t\t\t%s\r\n\n", fileToSend);
  printf("Waiting for kick, the firmware...\r\n");
  printf("Hit any key in case kick is not triggered automatically.\r\n");
  printf(MSG_STOP);
}



void goodbye(
  void
  )
{
  printf(MSG_START);
  headline();
  printf("Good-Bye.\r\n");
  printf(MSG_RESET);
}



int testflush_stdout(
  int mode,
  char c
  )
{
  static int permanent_flush = 0;
  int flush;

  flush = 0;
  if(mode == FLUSH_WITH_INPUT) {
    switch(c) {
      case ':':
      case '?':
        permanent_flush = 1;
        break;
    }
  }
  if(c == CLIM_NL) {
    flush = 1;
    permanent_flush = 0;
  }
  if(flush || permanent_flush)
    return ON;
  else
    return OFF;
}



int testflush_file(
  char *pc
  )
{
  if(*pc == CLIM_NL)
    return ON;
  else
    return OFF;
}



/******************************************************************//**
 * @brief
 * Endless Loop of Stream Monitoring.
 *
 * The following streams are monitored/accessed as necessary:
 * - stdin  Standard Input
 * - fdtty  Serial Port /dev/ttyS0
 * - fdrd   File, used for file to chip transmission
 * - fdwr   File, used for chip to file transmission
 * - stdout Standard Output
 * - stderr Standard Error Output
 *
 * In general, everything that is received from fdtty is send to stdout,
 * and everything that is received at stdin is send to fdtty. But not
 * everything, there are some control chars that trigger some more
 * actions...
 *
 * @param fdtty
 * /dev/ttyS0 File Descriptor
 * @return
 * Returns status of kick after exiting.
 *
 */
char polling(
  int fdtty
  )
{
  //tty exit sequence
  enum tag_step_exit {
    check_A = 0,
    check_B,
    exit_C
  } step_exit = check_A;

  //SOH data
  char file_format;
  char file_command;
  enum tag_SOH_step {
    SOH_get_command = 0,
    SOH_get_format
  } SOH_step;

  //send file to chip
  char a = 0;
  char b = 0;
  char errcode;
  enum tag_filestep tx;
  int wait_feedback;
  int flag_stx;
  int fdrd = -1;    //used to switch appropriate functionality on/off

  //send chip to file
  char c = 0;
  char * pc;
  int fdwr = -1;    //used to switch appropriate functionality on/off
  char spinwheel[] = {'\\', '-', '/', '|'};
  char* pspinwheel = spinwheel + sizeof(spinwheel) - 1;
  int flag_nodata;
  int dry_line = OFF;
  int line_retry;
  int retry_count;

  //protocol control
  char stx = STX;
  char etx = ETX;
  char eot = EOT;
  char ack = ACK;
  char can = CAN;
  int flag_can = OFF;
  char nak = NAK;
  char enq = ENQ;
  char clim = CLIM_HEXDUMP;

  //stream supervision
  fd_set rdfds;
  fd_set wrfds;
  int nfds;       //maximal file descriptor, see macro MAX()
  int r;          //return value for pselect(), read(), write(), ...
  const struct timespec tspec = { TIMEOUT_SEC, TIMEOUT_NSEC };

  //explicit stream switches and flags
  const int read_stdin = ON;
  const int read_tty = ON;
  int write_stdout = OFF;
  int write_tty = OFF;
  int write_file = OFF;
  int read_SOH = OFF;
  int screen_output;

  //tty read buffer
  unsigned char buf_rdtty[SIZE_BUF_RDTTY];
  int pwr_rdtty = 0;
  int prd_rdtty = 0;

  //stream buffer and pointers, stdout
  unsigned char buf_stdout[SIZE_BUF_STDOUT];
  int pwr_stdout = 0;
  int prd_stdout = 0;

  //stream buffer and pointers, file
  unsigned char buf_file[SIZE_BUF_FILE];
  int pwr_file = 0;
  int prd_file = 0;

  //HEXDUMP
  struct tag_hexdumpline hexdumpline = {
    OFF, OFF, '-'
  };
  //MOTBIN
  struct tag_motline motline;
  //chip to file
  int linepos_chip2file = -1;
  //file to chip
  int linepos_file2chip = -1;
  int ispayload_file2chip = 0;
  int isbyte_file;  //will be initialized at STX of line

  //setup trigger
  write_tty = ON;
  pc = &ack;

  while(ON) {
    //reset pointers
    if(prd_rdtty == pwr_rdtty)
      prd_rdtty = pwr_rdtty = 0;
    if(prd_stdout == pwr_stdout)
      prd_stdout = pwr_stdout = 0;
    if(prd_file == pwr_file)
      prd_file = pwr_file = 0;

    //is the read buffer empty?
    //do we need to write?
    if(prd_rdtty == pwr_rdtty || write_stdout || write_tty || write_file) {
      //clear
      nfds = 0;
      FD_ZERO(&rdfds);
      FD_ZERO(&wrfds);

      //write to stdout?
      if(write_stdout == ON) {
          FD_SET(STDOUT_FILENO, &wrfds);
          nfds = MAX(nfds, STDOUT_FILENO);
          write_stdout = OFF;   //automatic reset
      }

      //write to file?
      if(write_file == ON) {
        FD_SET(fdwr, &wrfds);
        nfds = MAX(nfds, fdwr);
        write_file = OFF;    //automatic reset
      }

      //write to tty?
      if(write_tty == ON) {
        FD_SET(fdtty, &wrfds);
        nfds = MAX(nfds, fdtty);
        write_tty = OFF;     //automatic reset
      }

      //read from tty?
      if((read_tty == ON) || (read_SOH == ON)) {
        FD_SET(fdtty, &rdfds);
        nfds = MAX(nfds, fdtty);
      }

      //read stdin?
      if(read_stdin == ON) {
        FD_SET(STDIN_FILENO, &rdfds);
        nfds = MAX(nfds, STDIN_FILENO);
      }

      //read from file?
      if(fdrd > -1) {
        if(wait_feedback == OFF) {
          FD_SET(fdrd, &rdfds);
          nfds = MAX(nfds, fdrd);
        }
      }

      //===============================================================
      //wait for pselect()
      /*
       * NOTE
       *
       * If all chars have been read into cbuf so far, we should proceed
       * reading the cbuf buffer instead of waiting for more chars on tty.
       * Because no char will be present, the following pselect() will
       * time out.
       *
       */
      r = pselect(nfds + 1, &rdfds, &wrfds, NULL, &tspec, NULL);

      //check pselect() error
      if(r == -1)
        EXITFAIL("pselect()");

      //check pselect() timeout
      if(!r) {
        if(fdwr > -1 || fdrd > -1) {
          /*
           * Transmission Error (Timeout)
           *
           * NOTE This timeout occurs most likely due to a fifo buffer overrun.
           *
           * The firmware 'kick' has sent ETX which 'connect' missed to
           * catch and which it keeps waiting for. In turn, 'kick'
           * is waiting for an ACK or NAK now... that's the trap.
           *
           * As 'kick' is using getChar(), it cannot monitor a timeout
           * like 'connect' has already done.
           *
           * In this situation, sending an ENQ to request the line once
           * more gets ourselfs out of the trap.
           *
           */
          if(!errcode)  //don't override a previous error
            errcode = ERRC__LINE_TIMEOUT;
          goto FAKE_ETX;
        }
        continue;
      }
      //===============================================================

      //write to stdout
      if(STDOUT_FILENO > -1) {
        if(FD_ISSET(STDOUT_FILENO, &wrfds)) {
          r = write(STDOUT_FILENO, buf_stdout + prd_stdout, pwr_stdout - prd_stdout);
          if(r < 1)
            EXITFAIL("write()");
          prd_stdout += r;
        }
      }

      //write to file
      if(fdwr > -1) {
        if(FD_ISSET(fdwr, &wrfds)) {
          r = write(fdwr, buf_file + prd_file, pwr_file - prd_file);
          if(r < 1)
            EXITFAIL("write()");
          prd_file += r;
        }
      }

      //write to tty
      if(fdtty > -1) {
        if(FD_ISSET(fdtty, &wrfds)) {
          r = write(fdtty, pc, 1);
          if(r < 1)
            EXITFAIL("write()");
        }
      }

      //read from stdin: any keystroke?
      if(STDIN_FILENO > -1) {
        if(FD_ISSET(STDIN_FILENO, &rdfds)) {
          r = read(STDIN_FILENO, &c, 1);
          if(r < 1)
            EXITFAIL("read()");
          else {
            //file to chip
            if(fdrd > -1) {
              switch(c) {
                case 'q':
                  flag_can = ON;
                  break;
              }
            }
            //chip to file
            else if(fdwr > -1) {
              switch(c) {
                case 'q':
                  flag_can = ON;
                  break;
              }
            }
            //idle
            else {
              write_tty = ON;
              continue;   //skip the rest
            }
          }
        }
      }

      //read from tty: any char from kick?
      if(fdtty > -1) {
        if(FD_ISSET(fdtty, &rdfds)) {
          r = read(fdtty, buf_rdtty + pwr_rdtty, SIZE_BUF_RDTTY - pwr_rdtty);

          if(r == -1 && errno == EINTR)
            continue;

          if(r == -1)
            EXITFAIL("read() into buf_rdtty");

          /*
           * NOTE
           * this is essential, otherwise we will exit if SIZE_BUF_RDTTY is
           * bigger than 1 but still small
           *
           */
          if(r == 0)
            continue;

          pwr_rdtty += r;
        }
      }

      //read from file: any file data to be processed?
      if(fdrd > -1) {
        if(FD_ISSET(fdrd, &rdfds)) {
          switch(tx) {
            case start_file_readout:
              a = 0;
              flag_stx = ON;

            case byte_preview:
              b = a;
              r = read(fdrd, &a, 1);
              if(r == -1)
                EXITFAIL ("read()");
              else if(!r) {
                a = EM;
                if(b == EM) {
                  tx = send_eot;
                  continue;
                }
              }

            case check_stx:
              if(flag_stx == ON) {
                flag_stx = OFF;
                errcode = ERRC__SUCCESS;  //reset for this line
                pc = &stx;
                write_tty = ON;
                tx = check_CRNL;
                wait_feedback = ON;
                if(file_format == MOTBIN) {
                  linepos_file2chip = 0;
                  isbyte_file = 0;  //b is zero, a is pointing to 'S'
                }
                continue;
              }

            case check_CRNL:
              if(b == CARR_RET) {
                if(a != NEW_LINE) {
                  if(file_format == HEXDUMP)
                    tx = send_clim;
                  else
                    tx = send_etx_or_can;
                }
                else
                  tx = byte_preview;
              }
              else if(b == NEW_LINE) {
                if(file_format == HEXDUMP)
                  tx = send_clim;
                else
                  tx = send_etx_or_can;
              }
              else
                tx = send_byte;
              continue;

            case send_byte:
              tx = byte_preview;
              if(b) {
                if((ispayload_file2chip = line_info(file_format, &linepos_file2chip, b, NULL, &motline, HEXMODE))) {
                  if((isbyte_file ^= 1)) {
                    b = (hexdigit2bin(b) << 4) | hexdigit2bin(a);
                    pc = &b;
                    write_tty = ON;
                    wait_feedback = ON;
                  }
                }
                else {
                  pc = &b;
                  write_tty = ON;
                  wait_feedback = ON;
                }
              }
              continue;

            case send_clim:
              pc = &clim;
              write_tty = ON;
              wait_feedback = ON;
              tx = send_etx_or_can;
              continue;

            case send_etx_or_can:
              if(flag_can) {
                flag_can = OFF;
                pc = &can;
              }
              else
                pc = &etx;
              write_tty = ON;
              tx = wait_char;
              linepos_file2chip = -1;
              continue;

            case wait_char:
            case receive_can:
            case receive_nak:
              continue;

            case receive_ack:
              flag_stx = ON;
              tx = byte_preview;
              continue;

            case send_eot:  //do we need wait_feedback = ON here?
              pc = &eot;
              write_tty = ON;
              tx = flush_buffers;
              continue;

            case flush_buffers:
              tx = close_tx;
              if(errcode != ERRC__SUCCESS) {
                WRITE_BUF_STDOUT('\n');
                write_stdout = screen_output;  //will be served first, thus flushing the buffer
                WRITE_BUF_FILE('\n');
                write_file = ON;    //will be served second
              }
              continue;

            case close_tx:
              close(fdrd);
              fdrd = -1;
              if(!screen_output)
                printf(" \r");  //clear remaining character of spinwheel
              put_error(errcode, 0);
              //reset buffer pointers
              //TODO: why is that necessary?
              prd_stdout = pwr_stdout = 0;
              prd_file = pwr_file = 0;
              continue;
          };
        }
      }
    }
    //is data in buffer?
    else if(prd_rdtty < pwr_rdtty) {
      //read from buf_rdtty
      c = buf_rdtty[prd_rdtty++];

      //catch error status
      if(tx == receive_nak) {
        errcode = c;
        tx = send_eot;
      }
      else {
        //scan for propeller-load exit sequence
        //don't scan when receiving random data
        //don't scan when receiving MOTBIN’s binary data
        if(!dry_line && \
            (linepos_chip2file < 0 && linepos_file2chip < 0) \
          ) {
          switch(step_exit) {
            case check_A:
              if((unsigned char)c == EXIT_CHAR_A) {
                step_exit++;
                continue;
              }
              break;

            case check_B:
              if(c == EXIT_CHAR_B) {
                step_exit++;
                continue;
              }
              break;

            case exit_C:
              return(c);
          }
          step_exit = check_A;    //reset
        }
        //file to chip (data and control chars)
        if(fdrd > -1) {
          if(c == *pc)
            wait_feedback = OFF;

          //binary data transmission (payload only, no control chars)
          if(ispayload_file2chip) {
            if(screen_output == ON) {
              char a, b;
              a = bin2hexdigit((*pc >> 4) & 0x0f, LOWER_CASE);
              b = bin2hexdigit(*pc & 0x0f, LOWER_CASE);
              WRITE_BUF_STDOUT(a);
              WRITE_BUF_STDOUT(b);
            }
          }
          //hexadecimal data transmission (control chars as well)
          else {
            switch(c) {
              case STX:
              case ENQ:
              case SOH:
              case EOT:
              case ETX:
                break;

              case ACK:
                if(tx == wait_char) {
                  tx = receive_ack;
                  if(!screen_output) {    //provide a spinning wheel if no output on screen available
                    WRITE_BUF_STDOUT(*pspinwheel);
                    if(pspinwheel == spinwheel)
                      pspinwheel = spinwheel + sizeof(spinwheel) - 1;
                    else
                      pspinwheel--;
                    WRITE_BUF_STDOUT('\b');
                    write_stdout = ON;
                  }
                }
                break;

              case NAK:
                if(tx == wait_char)
                  tx = receive_nak;
                break;

              case CAN:
                if(tx == wait_char)
                  tx = receive_can;
                break;

              default:
                if(isprint(c) || c == CLIM_CR || c == CLIM_NL) {  //do we need this 'security' feature?
                  if(screen_output == ON) {
                    WRITE_BUF_STDOUT(c);
                    write_stdout = testflush_stdout(FLUSH_NO_INPUT, c);
                  }
                }
                else {    //should we flag any error?
                }
            }
          }
          /*
           * Now, wait_feedback should be zero.
           * If not, proceed with wrong character and create timeout error.
           */
        }
        //chip to file (data and control chars)
        else if(fdwr > -1) {
          //binary data transmission (payload only, no control chars)
          if(line_info(file_format, &linepos_chip2file, c, &hexdumpline, &motline, BINMODE)) {
            unsigned char a, b;
            a = bin2hexdigit((c >> 4) & 0x0f, LOWER_CASE);
            b = bin2hexdigit(c & 0x0f, LOWER_CASE);
            if(screen_output == ON) {
              WRITE_BUF_STDOUT(a);
              WRITE_BUF_STDOUT(b);
            }
            WRITE_BUF_FILE(a);
            WRITE_BUF_FILE(b);
          }
          //hexadecimal data transmission (control chars as well)
          else {
            if(!dry_line) {   //don't scan for control characters when receiving random data
              switch(c) {
                case STX:
                  if(linepos_chip2file == -1) {
                    flag_nodata = 1;  //will be cleared with first valid data character
                    /*
                     * switch line_info() scanner on,
                     * will be switched off in case ETX,
                     * will be switched off in case of timeout error
                     */
                    linepos_chip2file = 0;
                    break;
                  }

                case EOT:
                  if(linepos_chip2file == -1) {
                    close(fdwr);
                    fdwr = -1;
                    write_file = OFF;
                    if(!errcode && (!screen_output || flag_nodata))
                      printf(" \r");  //clear remaining character of spinwheel
                    put_error(errcode, retry_count);
                    break;
                  }

                case ETX:
                  if(flag_can) {
                    flag_can = OFF;
                    errcode = ERRC__JOB_CANCELLATION;
                  }
                  else if(!errcode && !flag_nodata) {
                    switch(file_format) {
                      case MOTBIN:
                        if(motline.checksum != motline.chksum_rx)
                          errcode = ERRC__CHECKSUM_MISMATCH;
                        else if(linepos_chip2file != motline.length)
                          errcode = ERRC__LINE_LENGTH_MISMATCH;
                        break;
                      case HEXDUMP:
                        if(!hexdumpline.is_comment_line && (linepos_chip2file - 1) != HEXDUMP_LINELEN)
                          errcode = ERRC__LINE_LENGTH_MISMATCH;
                        break;
                    }
                  }
FAKE_ETX:
                  if(errcode < ERRC__CHECKSUM_MISMATCH && !flag_nodata) {
                    //start Write CLIM_NL
                    c = CLIM_NL;    //temporary use of c
                    WRITE_BUF_FILE(c);
                    write_file = testflush_file(&c);
                    //end Write CLIM_NL
                  }
                  linepos_chip2file = -1;   //switch line scanner off
                  switch(errcode) {
                    case ERRC__SUCCESS:
                      pc = &ack;
                      line_retry = LINE_RETRY;
                      if(!screen_output || flag_nodata) {    //provide a spinning wheel if no output on screen available
                        WRITE_BUF_STDOUT__MSG_START;
                        WRITE_BUF_STDOUT(*pspinwheel);
                        if(pspinwheel == spinwheel)
                          pspinwheel = spinwheel + sizeof(spinwheel) - 1;
                        else
                          pspinwheel--;
                        WRITE_BUF_STDOUT('\b');
                        WRITE_BUF_STDOUT__MSG_STOP;
                        write_stdout = ON;
                      }
                      break;

                    case ERRC__JOB_CANCELLATION:
                      pc = &can;
                      break;

                    case ERRC__CHECKSUM_MISMATCH:
                    case ERRC__LINE_LENGTH_MISMATCH:
                    case ERRC__OUT_OF_CHARSET:
                    case ERRC__LINE_TIMEOUT:
                      prd_stdout = pwr_stdout = 0;
                      prd_file = pwr_file = 0;
                      prd_rdtty = pwr_rdtty = 0;
                      dry_line = OFF;
                      if(line_retry-- == 0)
                        pc = &nak;
                      else {
#ifdef ERROR_MSG_READOUT
                        put_error(errcode, 0);
                        usleep(WAIT_AFTER_ERROR);
#endif
                        pc = &enq;
                        errcode = ERRC__SUCCESS;    //reset
                        retry_count++;
                      }
                      break;

                    default:  //any other error?
                      pc = &nak;
                      break;
                  }
                  write_tty = ON;
                  break;

                default:
                  //check charset
                  if((c == CLIM_CR) || (c == CLIM_NL))
                    goto LOC_WRITE;
                  if((file_format == MOTBIN) && ((c == 'S') || isxdigit(c)))
                    goto LOC_WRITE;
                  if(file_format == HEXDUMP) {
                    if(hexdumpline.is_charset_print && isprint(c))
                      goto LOC_WRITE;
                    if(!hexdumpline.is_charset_print && ((c == '#') || (c == ':') || (c == ' ') || isxdigit(c)))
                      goto LOC_WRITE;
                  }
                  errcode = ERRC__OUT_OF_CHARSET;
                  linepos_chip2file = -1;   //switch line scanner off
                  dry_line = ON;
                  break;

LOC_WRITE:
                  if(screen_output == ON) {
                    WRITE_BUF_STDOUT(c);
                    write_stdout = testflush_stdout(FLUSH_NO_INPUT, c);
                  }
                  //discard CLIM_NL here, write after ETX instead!
                  //this allows us to overwrite a broken line in case of line length mismatch
                  if(c != CLIM_NL) {
                    WRITE_BUF_FILE(c);
                    write_file = testflush_file(&c);
                  }
                  flag_nodata = 0;
                  break;
              }
            }
          }
        }
        //control commands: menu is active
        else {
          //SOH data
          if(read_SOH == ON) {
            switch(SOH_step) {
              case SOH_get_command:
                file_command = c;
                SOH_step = SOH_get_format;
                break;

              case SOH_get_format:
                file_format = c;
                read_SOH = OFF;
                errcode = ERRC__SUCCESS;   //reset
                flag_can = OFF;   //reset
                screen_output = ON;
                switch(file_command) {
                  case FILE_TO_CHIP_NOSCREEN:
                    screen_output = OFF;

                  case FILE_TO_CHIP:
                    //initiate transmission, stream will go active
                    fdrd = open(fileToSend, O_RDONLY);
                    if(fdrd == -1) {
                      //issue error message
                      errcode = ERRC__NO_SUCH_FILE;
                      put_error(errcode, 0);
                      //send EOT
                      pc = &eot;
                      write_tty = ON;
                      continue;
                    }
                    else {
                      tx = start_file_readout;
                      wait_feedback = OFF;
                      printf(MSG_START);
                      headline();
                      printf("sending data, reading %s...\r\n", fileToSend);
                      if(screen_output) //remove host color setting if spinning wheel is not present
                        printf(MSG_STOP);
                    }
                    break;

                  case CHIP_TO_FILE_NOSCREEN:
                    screen_output = OFF;

                  case CHIP_TO_FILE:
                    line_retry = LINE_RETRY;
                    retry_count = 0;
                    fdwr = open(fileToCatch, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
                    if(fdwr == -1) {
                       EXITFAIL("open()");
                      /*
                       * FIXME
                       * This error handling is very basic. It will leave the chipflasher board
                       * with its SPI-port active!
                       * It would be better if kick would now expect an ACK or NAK before it proceeds.
                       *
                       */
                    }
                    printf(MSG_START);
                    headline();
                    printf("receiving data, writing %s...\r\n", fileToCatch);
                    if(screen_output) //remove host color setting if spinning wheel is not present
                      printf(MSG_STOP);
                    break;
                }
                break;
            }
          }
          //normal menu
          else {
            switch(c) {
              case ACK:
              case NAK:
              case EOT:
              case STX:
              case ETX:
              case CAN:
                break;

              case ENQ:
                pc = &ack;
                write_tty = ON;
                continue;

              case SOH:   //start of heading for tx, rx
                SOH_step = SOH_get_command;
                read_SOH = ON;
                continue;

              default:  //all other chars
                pc = &c;
                if(isprint(c) || isspace(c)) {
                  WRITE_BUF_STDOUT(c);
                  write_stdout = testflush_stdout(FLUSH_WITH_INPUT, c);
                }
                continue;
            }
          }
        }
      }
    }
  }

  //this line should never be reached
  EXITFAIL("polling()");
}



void restore_stdout (
  void
  )
{
  tcsetattr(STDOUT_FILENO, TCSANOW, &old_stdio);
  tcsetattr(STDOUT_FILENO, TCSAFLUSH, &old_stdio);
}



int connect (
  char * dev,
  const speed_t baudrate,
  unsigned int bps
  )
{
  int fdtty;
  struct termios tio;
  struct termios stdio;

  //initialize memory
  memset(&stdio, 0, sizeof(stdio));
  memset(&tio, 0, sizeof(tio));

  //save current stdout terminal settings and restore later
  tcgetattr(STDOUT_FILENO, &old_stdio);

  //set exit function
  {
    int r;
    r = atexit(restore_stdout);
    if(r != 0) {
       fprintf(stderr, MSG_START"cannot set exit function"MSG_STOP"\n");
       exit(EXIT_FAILURE);
    }
  }

  //configure ttyS0
  tio.c_iflag = 0;              //specify input modes
  tio.c_oflag = 0;              //specify output modes
  tio.c_cflag = CS8|CREAD|CLOCAL;   //flag constants (8n1, see termios.h for more information)
  tio.c_lflag = 0;                  //local modes
  tio.c_cc[VMIN] = 0;               //minimum of input chars
  tio.c_cc[VTIME] = 0;              //input timeout in deciseconds

  /*
   * FIXME:
   * 1) Open() seems to initialize DTR and RTS to SPACE thus triggering
   *    a reset even if we don't want to. See Notes below.
   * 2) Open() seems to corrupt an existing locked connection.
   */
  if((fdtty = open(dev, O_RDWR|O_NONBLOCK)) != -1) {    //open port
    lock_tty(fdtty);
    cfsetospeed(&tio, baudrate);              //specify output speed (propeller default)
    cfsetispeed(&tio, baudrate);              //specify input speed (propeller default)
    tcsetattr(fdtty, TCSANOW, &tio);          //apply changes now

    /* Notes:
     *
     * The DTR signal is generated by your workstation and tells the computer or device on the
     * other end that you are ready (a space voltage) or not-ready (a mark voltage). DTR is
     * usually enabled automatically whenever you open the serial interface on the workstation.
     *
     * The RTS signal is set to the space voltage by your workstation to indicate that more data
     * is ready to be sent. Like CTS, RTS helps to regulate the flow of data between your
     * workstation and the computer or device on the other end of the serial cable. Most
     * workstations leave this signal set to the space voltage all the time.
     *
     * Source: https://www.cmrr.umn.edu/~strupp/serial.html#2_5_2
     */

    //configure stdout terminal
    stdio.c_iflag = 0;
    stdio.c_oflag = 0;
    stdio.c_cflag = 0;
    stdio.c_lflag = 0;
    stdio.c_cc[VMIN] = 1;
    stdio.c_cc[VTIME] = 0;
    tcsetattr(STDOUT_FILENO, TCSANOW, &stdio);
    tcsetattr(STDOUT_FILENO, TCSAFLUSH, &stdio);
    fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);  //make the reads non-blocking

    //disable board reset and trigger the propeller
    usleep(RESET_DURATION);
    set_DTR(fdtty, MARK);
    set_RTS(fdtty, MARK);

    //call polling
    polling(fdtty);

    //close '/dev/ttyS0'
    unlock_tty(fdtty);
    close(fdtty);

    //return status
    return EXIT_SUCCESS;
  }
  //return status
  return EXIT_FAILURE;
}



/******************************************************************//**
 * @brief
 * More flexible replacment for terminal setup by `propeller-load`.
 *
 * @param argc
 * Number of arguments.
 * @param **argv
 * Pointer to argument buffer.
 *
 * @return
 * Returns status as reported by the propeller board. Remember,
 * the exit sequence used by the propeller is: 0xff, 0x00, status.
 */
int main (
  int argc,
  char ** argv
  )
{
  char *port[] = {
    "/dev/ttyS0",
    "/dev/ttyS1",
    "/dev/ttyS2",
    "/dev/ttyS3",
    "/dev/ttyUSB0",
    "/dev/ttyUSB1"
  };

  struct tag_rate {
    const speed_t macro;
    const char * s;
    const unsigned int bps;   //should be 4 bytes
  } rate[] = {
    { B921600, "B921600", 921600 },
    { B576000, "B576000", 576000 },
    { B500000, "B500000", 500000 },
    { B460800, "B460800", 460800 },
    { B230400, "B230400", 230400 },
    { B115200, "B115200", 115200 },
    { B57600,  "B57600",   57600 },
    { B38400,  "B38400",   38400 }
  };

  int status;
  struct tag_rate * pRate;
  char * pPort;

  //set defaults
  pPort = port[PORT_DEFAULT];
  pRate = &(rate[RATE_DEFAULT]);

  switch(argc) {
    case 5:
      {
        int n;

        //filenames
        fileToCatch = argv[1];
        fileToSend = argv[2];

        //port
        n = SIZE_PORT;
        while(n--) {
          if(!strcmp(argv[3], port[n])) {
            pPort = port[n];
            break;
          }
        }

        //baudrate
        n = SIZE_RATE;
        while(n--) {
          if(!strcmp(argv[4], rate[n].s)) {
            pRate = &(rate[n]);
            break;
          }
        }
      }

    case 1:
      //standard action
      greeting(pPort, pRate->s);
      status = connect(pPort, pRate->macro, pRate->bps);
      if(status != EXIT_SUCCESS) {
        restore_stdout();
        put_error(ERRC__PORT_OPEN_FAILURE, 0);
      }
      goodbye();
      return status;

    default:
      //usage
      printf( "%s:\r\n", argv[0]);
      printf( "   version:\r\t\t" VERSION "\r\n");
      printf( "   usage:\r\t\t%s [chip2file file2chip port baudrate]\r\n\n", argv[0]);
      printf( "   chip2file:" \
              "\r\t\tA text file that will be used to store chip readouts\n" \
              "\r\t\tas lines of Motorola S-Record or Hex-Dump.\n" \
              "\r\t\tDefaults to '" FILE_TO_CATCH "'.\r\n");
      printf( "   file2chip:" \
              "\r\t\tA text file that contains data to be flashed as lines\n" \
              "\r\t\tof Motorola S-Record or Hex-Dump.\n" \
              "\r\t\tDefaults to '" FILE_TO_SEND "'.\r\n");
      printf( "   port:" \
              "\r\t\t/dev/ttyS0 (default) | /dev/ttyS1 | /dev/ttyS2 | /dev/ttyS3 |\n" \
              "\r\t\t/dev/ttyUSB0 | /dev/ttyUSB1\r\n");
      printf( "   baudrate:" \
              "\r\t\tB38400 | B57600 | B115200 (default) | B230400 | ... | B921600 \r\n");
      return 0; //should we flag an error?
  }
}
